@@ -103,7 +103,6 @@ class ConjectureRunner(object):
 
         self.health_check_state = None
 
-        self.used_examples_from_database = False
         self.tree = DataTree()
 
         self.best_observed_targets = defaultdict(lambda: NO_SCORE)
@@ -138,17 +137,26 @@ class ConjectureRunner(object):
         assert isinstance(data.observer, TreeRecordingObserver)
         self.call_count += 1
 
+        interrupted = False
         try:
             self.__stoppable_test_function(data)
+        except KeyboardInterrupt:
+            interrupted = True
+            raise
         except BaseException:
             self.save_buffer(data.buffer)
             raise
         finally:
-            data.freeze()
-            self.note_details(data)
+            # No branch, because if we're interrupted we always raise
+            # the KeyboardInterrupt, never continue to the code below.
+            if not interrupted:  # pragma: no branch
+                data.freeze()
+                self.note_details(data)
 
         self.debug_data(data)
 
+        assert len(data.buffer) <= BUFFER_SIZE
+
         if data.status >= Status.VALID:
             for k, v in data.target_observations.items():
                 self.best_observed_targets[k] = max(self.best_observed_targets[k], v)
@@ -430,8 +438,6 @@ class ConjectureRunner(object):
                     extra.sort(key=sort_key)
                     corpus.extend(extra)
 
-            self.used_examples_from_database = len(corpus) > 0
-
             for existing in corpus:
                 last_data = ConjectureData.for_buffer(
                     existing, observer=self.tree.new_observer()
@@ -660,7 +666,7 @@ class ConjectureRunner(object):
         Otherwise we call through to ``test_function``, and return a
         fresh result.
         """
-        buffer = hbytes(buffer)
+        buffer = hbytes(buffer)[:BUFFER_SIZE]
 
         def check_result(result):
             assert result is Overrun or (
@@ -694,7 +700,7 @@ class ConjectureRunner(object):
             data = self.new_conjecture_data_for_buffer(buffer)
             self.test_function(data)
             result = check_result(data.as_result())
-            assert status is None or result.status == status
+            assert status is None or result.status == status, (status, result.status)
             status = result.status
         if status == Status.OVERRUN:
             result = Overrun
@@ -846,20 +852,3 @@ def draw_bytes_with(prefix, parameter):
         return parameter.draw_bytes(n)
 
     return draw_bytes
-
-
-class DataClassification(Enum):
-    """When we generate some data and decide what we want to do about that it
-    can have one of three outcomes, and we want to make different decisions
-    based on that."""
-
-    # This data was actively bad in some way and we want to avoid generating
-    # things like it.
-    BAD = 1
-    # This data is basically fine. We don't really care about it one way or
-    # another.
-    NEUTRAL = 2
-
-    # This data exhibited some good behaviour that we would like to try to do
-    # more of.
-    GOOD = 3
